@@ -70,6 +70,7 @@ modified by
 
 #include <srt_compat.h> // SysStrError
 
+using namespace srt;
 using namespace srt::sync;
 
 namespace srt_logging {
@@ -97,7 +98,7 @@ const char* CUDTException::getErrorMessage() const ATR_NOTHROW
     return srt::strerror_get_message(m_iMajor, m_iMinor);
 }
 
-string CUDTException::getErrorString() const
+std::string CUDTException::getErrorString() const
 {
     return getErrorMessage();
 }
@@ -594,16 +595,20 @@ void LogDispatcher::CreateLogLinePrefix(std::ostringstream& serr)
 {
     using namespace std;
 
-    char tmp_buf[512];
+    SRT_STATIC_ASSERT(ThreadName::BUFSIZE >= sizeof("hh:mm:ss.") * 2, // multiply 2 for some margin
+                      "ThreadName::BUFSIZE is too small to be used for strftime");
+    char tmp_buf[ThreadName::BUFSIZE];
     if ( !isset(SRT_LOGF_DISABLE_TIME) )
     {
         // Not necessary if sending through the queue.
         timeval tv;
-        gettimeofday(&tv, 0);
+        gettimeofday(&tv, NULL);
         struct tm tm = SysLocalTime((time_t) tv.tv_sec);
 
-        strftime(tmp_buf, 512, "%X.", &tm);
-        serr << tmp_buf << setw(6) << setfill('0') << tv.tv_usec;
+        if (strftime(tmp_buf, sizeof(tmp_buf), "%X.", &tm))
+        {
+            serr << tmp_buf << setw(6) << setfill('0') << tv.tv_usec;
+        }
     }
 
     string out_prefix;
