@@ -157,6 +157,27 @@ TEST(SyncDuration, OperatorMultIntEq)
     EXPECT_EQ(count_milliseconds(a), 7000);
 }
 
+TEST(SyncRandom, GenRandomInt)
+{
+    vector<int> mn(64);
+
+    for (int i = 0; i < 2048; ++i)
+    {
+        const int rand_val = genRandomInt(0, 63);
+        ASSERT_GE(rand_val, 0);
+        ASSERT_LE(rand_val, 63);
+        ++mn[rand_val];
+    }
+
+    // Uncomment to see the distribution.
+    // for (size_t i = 0; i < mn.size(); ++i)
+    // {
+    //     cout << i << '\t';
+    //     for (int j=0; j<mn[i]; ++j) cout << '*';
+    //     cout << '\n';
+    // }
+}
+
 /*****************************************************************************/
 /*
  * TimePoint tests
@@ -428,8 +449,8 @@ TEST(SyncEvent, WaitForTwoNotifyOne)
     using wait_t = decltype(future_t().wait_for(chrono::microseconds(0)));
 
     wait_t wait_state[2] = {
-        move(future_result[0].wait_for(chrono::microseconds(100))),
-        move(future_result[1].wait_for(chrono::microseconds(100)))
+        move(future_result[0].wait_for(chrono::microseconds(500))),
+        move(future_result[1].wait_for(chrono::microseconds(500)))
     };
 
     cerr << "SyncEvent::WaitForTwoNotifyOne: NOTIFICATION came from " << notified_clients.size()
@@ -440,7 +461,7 @@ TEST(SyncEvent, WaitForTwoNotifyOne)
 
     // Now exactly one waiting thread should become ready
     // Error if: 0 (none ready) or 2 (both ready, while notify_one was used)
-    ASSERT_EQ(notified_clients.size(), 1);
+    ASSERT_EQ(notified_clients.size(), 1U);
 
     const int ready = notified_clients[0];
     const int not_ready = (ready + 1) % 2;
@@ -590,7 +611,7 @@ TEST(Sync, FormatTime)
         const regex rex("([[:digit:]]+D )?([[:digit:]]{2}):([[:digit:]]{2}):([[:digit:]]{2}).([[:digit:]]{6,}) \\[STDY\\]");
         std::smatch sm;
         EXPECT_TRUE(regex_match(timestr, sm, rex));
-        EXPECT_LE(sm.size(), 6);
+        EXPECT_LE(sm.size(), 6U);
         if (sm.size() != 6 && sm.size() != 5)
             return 0;
 
@@ -634,7 +655,7 @@ TEST(Sync, FormatTimeSys)
         const regex rex("([[:digit:]]{2}):([[:digit:]]{2}):([[:digit:]]{2}).([[:digit:]]{6}) \\[SYST\\]");
         std::smatch sm;
         EXPECT_TRUE(regex_match(timestr, sm, rex));
-        EXPECT_EQ(sm.size(), 5);
+        EXPECT_EQ(sm.size(), 5U);
         if (sm.size() != 5)
             return 0;
 
