@@ -20,6 +20,7 @@
 #include <map>
 #include <chrono>
 #include <thread>
+#include <atomic>
 #include <srt.h>
 #if !defined(_WIN32)
 #include <sys/ioctl.h>
@@ -48,8 +49,8 @@ using srt_logging::SockStatusStr;
 using srt_logging::MemberStatusStr;
 #endif
 
-volatile bool transmit_throw_on_interrupt = false;
-volatile bool transmit_int_state = false;
+std::atomic<bool> transmit_throw_on_interrupt {false};
+std::atomic<bool> transmit_int_state {false};
 int transmit_bw_report = 0;
 unsigned transmit_stats_report = 0;
 size_t transmit_chunk_size = SRT_LIVE_DEF_PLSIZE;
@@ -2259,7 +2260,7 @@ MediaPacket SrtSource::Read(size_t chunk)
 {
     static size_t counter = 1;
 
-    bool have_group ATR_UNUSED = !m_group_nodes.empty();
+    bool have_group SRT_ATR_UNUSED = !m_group_nodes.empty();
 
     bytevector data(chunk);
     // EXPERIMENTAL
@@ -2715,276 +2716,238 @@ static inline bool IsMulticast(in_addr adr)
     return c >= 224 && c <= 239;
 }
 
-
-class UdpCommon
+void UdpCommon::Setup(string host, int port, map<string,string> attr)
 {
-protected:
-    int m_sock = -1;
-    sockaddr_any sadr;
-    string adapter;
-    map<string, string> m_options;
+    m_sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+    if (m_sock == -1)
+        Error(SysError(), "UdpCommon::Setup: socket");
 
-    void Setup(string host, int port, map<string,string> attr)
+    int yes = 1;
+    ::setsockopt(m_sock, SOL_SOCKET, SO_REUSEADDR, (const char*)&yes, sizeof yes);
+
+    sadr = CreateAddr(host, port);
+
+    bool is_multicast = false;
+    if (sadr.family() == AF_INET)
     {
-        m_sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
-        if (m_sock == -1)
-            Error(SysError(), "UdpCommon::Setup: socket");
-
-        int yes = 1;
-        ::setsockopt(m_sock, SOL_SOCKET, SO_REUSEADDR, (const char*)&yes, sizeof yes);
-
-        sadr = CreateAddr(host, port);
-
-        bool is_multicast = false;
-        if (sadr.family() == AF_INET)
+        if (attr.count("multicast"))
         {
-            if (attr.count("multicast"))
+            if (!IsMulticast(sadr.sin.sin_addr))
             {
-                if (!IsMulticast(sadr.sin.sin_addr))
-                {
-                    throw std::runtime_error("UdpCommon: requested multicast for a non-multicast-type IP address");
-                }
-                is_multicast = true;
+                throw std::runtime_error("UdpCommon: requested multicast for a non-multicast-type IP address");
             }
-            else if (IsMulticast(sadr.sin.sin_addr))
+            is_multicast = true;
+        }
+        else if (IsMulticast(sadr.sin.sin_addr))
+        {
+            is_multicast = true;
+        }
+
+        if (is_multicast)
+        {
+            ip_mreq_source mreq_ssm;
+            ip_mreq mreq;
+            sockaddr_any maddr;
+            int opt_name;
+            void* mreq_arg_ptr;
+            socklen_t mreq_arg_size;
+
+            adapter = attr.count("adapter") ? attr.at("adapter") : string();
+            if (adapter == "")
             {
-                is_multicast = true;
+                Verb() << "Multicast: home address: INADDR_ANY:" << port;
+                maddr.sin.sin_family = AF_INET;
+                maddr.sin.sin_addr.s_addr = htonl(INADDR_ANY);
+                maddr.sin.sin_port = htons(port); // necessary for temporary use
+            }
+            else
+            {
+                Verb() << "Multicast: home address: " << adapter << ":" << port;
+                maddr = CreateAddr(adapter, port);
             }
 
-            if (is_multicast)
+            if (attr.count("source"))
             {
-                ip_mreq_source mreq_ssm;
-                ip_mreq mreq;
-                sockaddr_any maddr;
-                int opt_name;
-                void* mreq_arg_ptr;
-                socklen_t mreq_arg_size;
-
-                adapter = attr.count("adapter") ? attr.at("adapter") : string();
-                if (adapter == "")
-                {
-                    Verb() << "Multicast: home address: INADDR_ANY:" << port;
-                    maddr.sin.sin_family = AF_INET;
-                    maddr.sin.sin_addr.s_addr = htonl(INADDR_ANY);
-                    maddr.sin.sin_port = htons(port); // necessary for temporary use
-                }
-                else
-                {
-                    Verb() << "Multicast: home address: " << adapter << ":" << port;
-                    maddr = CreateAddr(adapter, port);
-                }
-
-                if (attr.count("source"))
-                {
-                    /* this is an ssm.  we need to use the right struct and opt */
-                    opt_name = IP_ADD_SOURCE_MEMBERSHIP;
-                    mreq_ssm.imr_multiaddr.s_addr = sadr.sin.sin_addr.s_addr;
-                    mreq_ssm.imr_interface.s_addr = maddr.sin.sin_addr.s_addr;
-                    inet_pton(AF_INET, attr.at("source").c_str(), &mreq_ssm.imr_sourceaddr);
-                    mreq_arg_size = sizeof(mreq_ssm);
-                    mreq_arg_ptr = &mreq_ssm;
-                }
-                else
-                {
-                    opt_name = IP_ADD_MEMBERSHIP;
-                    mreq.imr_multiaddr.s_addr = sadr.sin.sin_addr.s_addr;
-                    mreq.imr_interface.s_addr = maddr.sin.sin_addr.s_addr;
-                    mreq_arg_size = sizeof(mreq);
-                    mreq_arg_ptr = &mreq;
-                }
+                /* this is an ssm.  we need to use the right struct and opt */
+                opt_name = IP_ADD_SOURCE_MEMBERSHIP;
+                mreq_ssm.imr_multiaddr.s_addr = sadr.sin.sin_addr.s_addr;
+                mreq_ssm.imr_interface.s_addr = maddr.sin.sin_addr.s_addr;
+                inet_pton(AF_INET, attr.at("source").c_str(), &mreq_ssm.imr_sourceaddr);
+                mreq_arg_size = sizeof(mreq_ssm);
+                mreq_arg_ptr = &mreq_ssm;
+            }
+            else
+            {
+                opt_name = IP_ADD_MEMBERSHIP;
+                mreq.imr_multiaddr.s_addr = sadr.sin.sin_addr.s_addr;
+                mreq.imr_interface.s_addr = maddr.sin.sin_addr.s_addr;
+                mreq_arg_size = sizeof(mreq);
+                mreq_arg_ptr = &mreq;
+            }
 
 #ifdef _WIN32
-                const char* mreq_arg = (const char*)mreq_arg_ptr;
-                const auto status_error = SOCKET_ERROR;
+            const char* mreq_arg = (const char*)mreq_arg_ptr;
+            const auto status_error = SOCKET_ERROR;
 #else
-                const void* mreq_arg = mreq_arg_ptr;
-                const auto status_error = -1;
+            const void* mreq_arg = mreq_arg_ptr;
+            const auto status_error = -1;
 #endif
 
 #if defined(_WIN32) || defined(__CYGWIN__)
-                // On Windows it somehow doesn't work when bind()
-                // is called with multicast address. Write the address
-                // that designates the network device here.
-                // Also, sets port sharing when working with multicast
-                sadr = maddr;
-                int reuse = 1;
-                int shareAddrRes = setsockopt(m_sock, SOL_SOCKET, SO_REUSEADDR, reinterpret_cast<const char*>(&reuse), sizeof(reuse));
-                if (shareAddrRes == status_error)
-                {
-                    throw runtime_error("marking socket for shared use failed");
-                }
-                Verb() << "Multicast(Windows): will bind to home address";
-#else
-                Verb() << "Multicast(POSIX): will bind to IGMP address: " << host;
-#endif
-                int res = setsockopt(m_sock, IPPROTO_IP, opt_name, mreq_arg, mreq_arg_size);
-
-                if (res == status_error)
-                {
-                    Error(errno, "adding to multicast membership failed");
-                }
-
-                attr.erase("multicast");
-                attr.erase("adapter");
-            }
-        }
-
-        // The "ttl" options is handled separately, it maps to both IP_TTL
-        // and IP_MULTICAST_TTL so that TTL setting works for both uni- and multicast.
-        if (attr.count("ttl"))
-        {
-            int ttl = stoi(attr.at("ttl"));
-            int res = setsockopt(m_sock, IPPROTO_IP, IP_TTL, (const char*)&ttl, sizeof ttl);
-            if (res == -1)
-                Verb() << "WARNING: failed to set 'ttl' (IP_TTL) to " << ttl;
-            res = setsockopt(m_sock, IPPROTO_IP, IP_MULTICAST_TTL, (const char*)&ttl, sizeof ttl);
-            if (res == -1)
-                Verb() << "WARNING: failed to set 'ttl' (IP_MULTICAST_TTL) to " << ttl;
-
-            attr.erase("ttl");
-        }
-
-        m_options = attr;
-
-        for (auto o: udp_options)
-        {
-            // Ignore "binding" - for UDP there are no post options.
-            if (m_options.count(o.name))
+            // On Windows it somehow doesn't work when bind()
+            // is called with multicast address. Write the address
+            // that designates the network device here.
+            // Also, sets port sharing when working with multicast
+            sadr = maddr;
+            int reuse = 1;
+            int shareAddrRes = setsockopt(m_sock, SOL_SOCKET, SO_REUSEADDR, reinterpret_cast<const char*>(&reuse), sizeof(reuse));
+            if (shareAddrRes == status_error)
             {
-                string value = m_options.at(o.name);
-                bool ok = o.apply<SocketOption::SYSTEM>(m_sock, value);
-                if (!ok)
-                    Verb() << "WARNING: failed to set '" << o.name << "' to " << value;
+                throw runtime_error("marking socket for shared use failed");
             }
+            Verb() << "Multicast(Windows): will bind to home address";
+#else
+            Verb() << "Multicast(POSIX): will bind to IGMP address: " << host;
+#endif
+            int res = setsockopt(m_sock, IPPROTO_IP, opt_name, mreq_arg, mreq_arg_size);
+
+            if (res == status_error)
+            {
+                Error(errno, "adding to multicast membership failed");
+            }
+
+            attr.erase("multicast");
+            attr.erase("adapter");
         }
     }
 
-    void Error(int err, string src)
+    // The "ttl" options is handled separately, it maps to both IP_TTL
+    // and IP_MULTICAST_TTL so that TTL setting works for both uni- and multicast.
+    if (attr.count("ttl"))
     {
-        char buf[512];
-        string message = SysStrError(err, buf, 512u);
+        int ttl = stoi(attr.at("ttl"));
+        int res = setsockopt(m_sock, IPPROTO_IP, IP_TTL, (const char*)&ttl, sizeof ttl);
+        if (res == -1)
+            Verb() << "WARNING: failed to set 'ttl' (IP_TTL) to " << ttl;
+        res = setsockopt(m_sock, IPPROTO_IP, IP_MULTICAST_TTL, (const char*)&ttl, sizeof ttl);
+        if (res == -1)
+            Verb() << "WARNING: failed to set 'ttl' (IP_MULTICAST_TTL) to " << ttl;
 
-        if (Verbose::on)
-            Verb() << "FAILURE\n" << src << ": [" << err << "] " << message;
-        else
-            cerr << "\nERROR #" << err << ": " << message << endl;
-
-        throw TransmissionError("error: " + src + ": " + message);
+        attr.erase("ttl");
     }
 
-    ~UdpCommon()
+    m_options = attr;
+
+    for (auto o: udp_options)
     {
+        // Ignore "binding" - for UDP there are no post options.
+        if (m_options.count(o.name))
+        {
+            string value = m_options.at(o.name);
+            bool ok = o.apply<SocketOption::SYSTEM>(m_sock, value);
+            if (!ok)
+                Verb() << "WARNING: failed to set '" << o.name << "' to " << value;
+        }
+    }
+}
+
+void UdpCommon::Error(int err, string src)
+{
+    char buf[512];
+    string message = SysStrError(err, buf, 512u);
+
+    if (Verbose::on)
+        Verb() << "FAILURE\n" << src << ": [" << err << "] " << message;
+    else
+        cerr << "\nERROR #" << err << ": " << message << endl;
+
+    throw TransmissionError("error: " + src + ": " + message);
+}
+
+UdpCommon::~UdpCommon()
+{
 #ifdef _WIN32
-        if (m_sock != -1)
-        {
-            shutdown(m_sock, SD_BOTH);
-            closesocket(m_sock);
-            m_sock = -1;
-        }
+    if (m_sock != -1)
+    {
+        shutdown(m_sock, SD_BOTH);
+        closesocket(m_sock);
+        m_sock = -1;
+    }
 #else
-        close(m_sock);
+    close(m_sock);
 #endif
-    }
-};
+}
 
-
-class UdpSource: public virtual Source, public virtual UdpCommon
+UdpSource::UdpSource(string host, int port, const map<string,string>& attr)
 {
-    bool eof = true;
-public:
+    Setup(host, port, attr);
+    int stat = ::bind(m_sock, sadr.get(), sadr.size());
+    if (stat == -1)
+        Error(SysError(), "Binding address for UDP");
+    eof = false;
+    struct timeval tv;
+    tv.tv_sec = 1;
+    tv.tv_usec = 0;
+    if (::setsockopt(m_sock, SOL_SOCKET, SO_RCVTIMEO, (const char*) &tv, sizeof(tv)) < 0)
+        Error(SysError(), "Setting timeout for UDP");
+}
 
-    UdpSource(string host, int port, const map<string,string>& attr)
-    {
-        Setup(host, port, attr);
-        int stat = ::bind(m_sock, sadr.get(), sadr.size());
-        if (stat == -1)
-            Error(SysError(), "Binding address for UDP");
-        eof = false;
-        struct timeval tv;
-        tv.tv_sec = 1;
-        tv.tv_usec = 0;
-        if (::setsockopt(m_sock, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv)) < 0)
-            Error(SysError(), "Setting timeout for UDP");
-    }
-
-    MediaPacket Read(size_t chunk) override
-    {
-        bytevector data(chunk);
-        sockaddr_any sa(sadr.family());
-        int64_t srctime = 0;
+MediaPacket UdpSource::Read(size_t chunk)
+{
+    bytevector data(chunk);
+    sockaddr_any sa(sadr.family());
+    int64_t srctime = 0;
 AGAIN:
-        int stat = recvfrom(m_sock, data.data(), (int) chunk, 0, sa.get(), &sa.syslen());
-        int err = SysError();
-        if (transmit_use_sourcetime)
-        {
-            srctime = srt_time_now();
-        }
-        if (stat == -1)
-        {
-            if (!::transmit_int_state && err == SysAGAIN)
-                goto AGAIN;
+    int stat = recvfrom(m_sock, data.data(), (int) chunk, 0, sa.get(), &sa.syslen());
+    int err = SysError();
+    if (transmit_use_sourcetime)
+    {
+        srctime = srt_time_now();
+    }
+    if (stat == -1)
+    {
+        if (!::transmit_int_state && err == SysAGAIN)
+            goto AGAIN;
 
-            Error(SysError(), "UDP Read/recvfrom");
-        }
-
-        if (stat < 1)
-        {
-            eof = true;
-            return bytevector();
-        }
-
-        chunk = size_t(stat);
-        if (chunk < data.size())
-            data.resize(chunk);
-
-        return MediaPacket(data, srctime);
+        Error(SysError(), "UDP Read/recvfrom");
     }
 
-    bool IsOpen() override { return m_sock != -1; }
-    bool End() override { return eof; }
-};
+    if (stat < 1)
+    {
+        eof = true;
+        return bytevector();
+    }
 
-class UdpTarget: public virtual Target, public virtual UdpCommon
+    chunk = size_t(stat);
+    if (chunk < data.size())
+        data.resize(chunk);
+
+    return MediaPacket(data, srctime);
+}
+
+UdpTarget::UdpTarget(string host, int port, const map<string,string>& attr)
 {
-public:
-    UdpTarget(string host, int port, const map<string,string>& attr )
+    Setup(host, port, attr);
+    if (adapter != "")
     {
-        Setup(host, port, attr);
-        if (adapter != "")
-        {
-            auto maddr = CreateAddr(adapter, 0);
-            in_addr addr = maddr.sin.sin_addr;
+        auto maddr = CreateAddr(adapter, 0);
+        in_addr addr = maddr.sin.sin_addr;
 
-            int res = setsockopt(m_sock, IPPROTO_IP, IP_MULTICAST_IF, reinterpret_cast<const char*>(&addr), sizeof(addr));
-            if (res == -1)
-            {
-                Error(SysError(), "setsockopt/IP_MULTICAST_IF: " + adapter);
-            }
+        int res = setsockopt(m_sock, IPPROTO_IP, IP_MULTICAST_IF, reinterpret_cast<const char*>(&addr), sizeof(addr));
+        if (res == -1)
+        {
+            Error(SysError(), "setsockopt/IP_MULTICAST_IF: " + adapter);
         }
     }
+}
 
-    void Write(const MediaPacket& data) override
-    {
-        int stat = sendto(m_sock, data.payload.data(), data.payload.size(), 0, (sockaddr*)&sadr, sizeof sadr);
-        if (stat == -1)
-            Error(SysError(), "UDP Write/sendto");
-    }
-
-    bool IsOpen() override { return m_sock != -1; }
-    bool Broken() override { return false; }
-};
-
-class UdpRelay: public Relay, public UdpSource, public UdpTarget
+void UdpTarget::Write(const MediaPacket& data)
 {
-public:
-    UdpRelay(string host, int port, const map<string,string>& attr):
-        UdpSource(host, port, attr),
-        UdpTarget(host, port, attr)
-    {
-    }
+    int stat = sendto(m_sock, data.payload.data(), data.payload.size(), 0, (sockaddr*)&sadr, sizeof sadr);
+    if (stat == -1)
+        Error(SysError(), "UDP Write/sendto");
+}
 
-    bool IsOpen() override { return m_sock != -1; }
-};
 
 template <class Iface> struct Udp;
 template <> struct Udp<Source> { typedef UdpSource type; };
